/*
 * Copyright (c) 2013, SRI International
 * All rights reserved.
 * Licensed under the The BSD 3-Clause License;
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 * 
 * http://opensource.org/licenses/BSD-3-Clause
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of the aic-expresso nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.sri.ai.grinder.library;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.google.common.annotations.Beta;
import com.sri.ai.expresso.api.Expression;
import com.sri.ai.expresso.helper.Expressions;
import com.sri.ai.grinder.api.RewritingProcess;
import com.sri.ai.grinder.library.boole.And;

/**
 * A class providing unification of sub-expressions.
 * 
 * @author braz
 */
@Beta
public class Unification {

	public static Expression unificationCondition(Expression expression1, Expression expression2, RewritingProcess process) {
//		if (
//				(expression1 instanceof Symbol && expression2 instanceof CompoundSyntaxTree)
//				||
//				(expression1 instanceof CompoundSyntaxTree && expression2 instanceof Symbol)
//		) {
//			return Expressions.FALSE;
//		}
		
		if (expression1.numberOfArguments() != expression2.numberOfArguments()) {
			return Expressions.FALSE;
		}
		
		Expression functorOrSymbol1 = expression1.getFunctorOrSymbol();
		Expression functorOrSymbol2 = expression2.getFunctorOrSymbol();
		boolean functorOrSymbol1IsConstant = process.isConstant(functorOrSymbol1);
		boolean functorOrSymbol2IsConstant = process.isConstant(functorOrSymbol2);
		List<Expression> conjuncts = new LinkedList<Expression>();
		// If they are the same, the functor constraint is just true, 
		// so does not need to be taken.
		if (!functorOrSymbol1.equals(functorOrSymbol2)) {
			if (functorOrSymbol1IsConstant && functorOrSymbol2IsConstant) {
				return Expressions.FALSE;
			}
			else {
				conjuncts.add(Expressions.apply("=", functorOrSymbol1, functorOrSymbol2));
			}
		}

		Iterator<Expression> expression1ArgumentsIterator = expression1.getArguments().iterator();
		Iterator<Expression> expression2ArgumentsIterator = expression2.getArguments().iterator();
		while (expression1ArgumentsIterator.hasNext()) {
			Expression argument1 = expression1ArgumentsIterator.next();
			Expression argument2 = expression2ArgumentsIterator.next();
			Expression subCondition = unificationCondition(argument1, argument2, process);
			if (subCondition.equals(Expressions.FALSE)) {
				return Expressions.FALSE;
			}
			conjuncts.add(subCondition);
		}
		Expression result = And.make(conjuncts);
		return result;
	}
}
